#region

using System;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Net;
using System.Text.RegularExpressions;
using System.Windows.Forms;
using System.Xml.Serialization;
using EveMapSettings;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;
using libEveMapNG.RestAPI;
using Store = libEveMapNG.Data.Dynamic.DynamicStore;

#endregion

namespace EVEMapNG.SubForms
{
    public partial class ManageJumpbridges : Form
    {
        public ManageJumpbridges()
        {
            InitializeComponent();
        }

        private void ManageJumpbridges_Load(object sender, EventArgs e)
        {
            UpdateListView();
        }

        private void UpdateListView()
        {
            lvJumpBridges.Items.Clear();
            ListViewItem lvi;
            foreach (JumpBridge bridge in Store.Instance.JumpBridges)
            {
                SolarSystem FromSystem = bridge.From;
                SolarSystem ToSystem = bridge.To;
                lvi = new ListViewItem(FromSystem.Name);
                lvi.Tag = bridge;
                lvi.SubItems.Add(String.Format("Planet {0} - Moon {1}", bridge.Base.Planet, bridge.Base.Moon));
                lvi.SubItems.Add(ToSystem.Name);
                lvi.SubItems.Add(String.Format("Planet {0} - Moon {1}", bridge.Destination.Planet,
                                               bridge.Destination.Moon));
                lvJumpBridges.Items.Add(lvi);
            }
        }

        private void btnAdd_Click(object sender, EventArgs e)
        {
            if (string.IsNullOrEmpty(ssbToSystemSearch.Text) || String.IsNullOrEmpty(ssbFromSystemSearch.Text))
            {
                MessageBox.Show("The system to and from fields cannot be empty!");
                return;
            }

            if (string.IsNullOrEmpty(mtbFromPlanetMoon.Text) || String.IsNullOrEmpty(mtbFromPlanetMoon.Text))
            {
                MessageBox.Show("The starbase planet - moon fields cannot be empty!");
                return;
            }


            string[] fromPos = mtbFromPlanetMoon.Text.Split('-');
            string[] toPos = mtbToPlanetMoon.Text.Split('-');
            int fromSys, toSys;

            try
            {
                fromSys = SolarSystem.GetIDByName(ssbFromSystemSearch.Text);
                toSys = SolarSystem.GetIDByName(ssbToSystemSearch.Text);
            }
            catch
            {
                MessageBox.Show(
                    "Either your from system or to system doesn't exist.\r\n  I'm clearing both entries out so you can try again.\r\n  Please get it right this time.  >:[");
                ssbFromSystemSearch.Text = string.Empty;
                ssbToSystemSearch.Text = string.Empty;
                return;
            }

            int fromPlanet;
            int fromMoon;
            int toPlanet;
            int toMoon;
            if (!(int.TryParse(fromPos[0], out fromPlanet) &&
                  int.TryParse(fromPos[1], out fromMoon) &&
                  int.TryParse(toPos[0], out toPlanet) &&
                  int.TryParse(toPos[1], out toMoon)))
            {
                MessageBox.Show("Please enter the planet and moon of the jumpbridge starbases.");
                return;
            }

            JumpBridge bridge = new JumpBridge
                                    {
                                        From = SolarSystem.GetByID(fromSys),
                                        To = SolarSystem.GetByID(toSys),
                                        Base = new JumpBridgePos
                                                   {
                                                       Planet = fromPlanet,
                                                       Moon = fromMoon,
                                                   },
                                        Destination = new JumpBridgePos
                                                          {
                                                              Planet = toPlanet,
                                                              Moon = toMoon,
                                                          },
                                        BridgeLine = Color.Black
                                    };
            //This is a hack until we can figure out a better way
            DynamicStore.Instance.JumpBridges.Add(bridge);
            mtbToPlanetMoon.Clear();
            mtbFromPlanetMoon.Clear();
            ssbToSystemSearch.Text = null;
            ssbFromSystemSearch.Text = null;
            UpdateListView();
        }

        private void btnRemove_Click(object sender, EventArgs e)
        {
            if (lvJumpBridges.SelectedIndices.Count != 1)
                return;
            JumpBridge bridge = (JumpBridge) lvJumpBridges.SelectedItems[0].Tag;
            DynamicStore.Instance.JumpBridges.Remove(bridge);
            UpdateListView();
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            DialogResult result = MessageBox.Show(this,
                                                  "Are you sure you want to clear the list?  All jumpbridges will be removed.",
                                                  "Clear Jumpbridges",
                                                  MessageBoxButtons.OKCancel, MessageBoxIcon.Warning,
                                                  MessageBoxDefaultButton.Button2);
            if (result == DialogResult.OK)
            {
                DynamicStore.Instance.JumpBridges.Clear();
                UpdateListView();
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            DynamicStore.WriteToFile();
            Close();
        }

        private void lnkJumpbridgeWikiPage_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("http://code.google.com/p/evemap/wiki/JumpbridgeImportGuide");
        }

        private void btnImportJumpbridges_Click(object sender, EventArgs e)
        {
            if (!Clipboard.ContainsText())
            {
                MessageBox.Show("Please copy the jumpbridge tables in the wiki before you push button.", Text,
                                MessageBoxButtons.OK, MessageBoxIcon.Warning);
                return;
            }
            string clipboardContents = Clipboard.GetText();
            Regex rGetJumpBridges =
                new Regex(
                    @"(?<fromSys>[a-zA-Z0-9\-]+) @ (?<fromPlanet>\d+)-(?<fromMoon>\d+)(\t| )(?<toSys>[a-zA-Z0-9\-]+) @ (?<toPlanet>\d+)-(?<toMoon>\d+)(\t| )(?<Status>[a-zA-Z]+)(\t| )(?<Corporation>[a-z A-Z0-9]+)(\t| )(?<Password>[a-zA-Z0-9]+)(\t| )(?<Distance>[.0-9]+)(\t| )(?<Color>[a-zA-z]+)",
                    RegexOptions.ExplicitCapture);
            DynamicStore.Instance.JumpBridges.Clear();
            foreach (Match m in rGetJumpBridges.Matches(clipboardContents))
            {
                if (m.Groups["Status"].ToString() == "Offline")
                    continue;
                int fromSys, toSys;
                try
                {
                    fromSys = SolarSystem.GetIDByName(m.Groups["fromSys"].Value);
                    toSys = SolarSystem.GetIDByName(m.Groups["toSys"].Value);
                }
                catch (Exception exception)
                {
                    MessageBox.Show(
                        String.Format(
                            "Failed to add jump bridge, {0}. Please fix the wiki.",
                            exception.Message), "Invalid Solar System", MessageBoxButtons.OK, MessageBoxIcon.Error);
                    continue;
                }

                int fromPlanet = Int32.Parse(m.Groups["fromPlanet"].Value);
                int toPlanet = Int32.Parse(m.Groups["toPlanet"].Value);
                int fromMoon = Int32.Parse(m.Groups["fromMoon"].Value);
                int toMoon = Int32.Parse(m.Groups["toMoon"].Value);
                Color bridgeColor = Color.FromName(m.Groups["Color"].Value);
                string corporation = m.Groups["Corporation"].Value;
                string password = m.Groups["Password"].Value;

                JumpBridge bridge = new JumpBridge
                                        {
                                            From = SolarSystem.GetByID(fromSys),
                                            To = SolarSystem.GetByID(toSys),
                                            Base = new JumpBridgePos
                                                       {
                                                           Planet = fromPlanet,
                                                           Moon = fromMoon,
                                                       },
                                            Destination = new JumpBridgePos
                                                              {
                                                                  Planet = toPlanet,
                                                                  Moon = toMoon,
                                                              },
                                            BridgeLine = bridgeColor,
                                            Corporation = corporation,
                                            Password = password,
                                        };
                //This is a hack until we can figure out a better way
                DynamicStore.Instance.JumpBridges.Add(bridge);
            }
            UpdateListView();
        }

        private void btnImportGoonFleetBridges_Click(object sender, EventArgs e)
        {
            Cursor regularCursor = Cursor;
            Cursor = Cursors.WaitCursor;
            Application.DoEvents();

            DynamicStore.Instance.JumpBridges.Clear();

            //Get the XML from somewhere
            WebClient myClient = new WebClient();
            Stream apiXML = myClient.OpenRead(UserSettings.Instance.RestJumpbridgeUrl);

            BridgeList myBridges = (BridgeList) new XmlSerializer(typeof (BridgeList)).Deserialize(apiXML);

            foreach (Bridge apibridge in myBridges.Bridges)
            {
                //Fixups for the broken API data
                //I guess they dont have the system id's for the drone regions?
                if (apibridge.From.SystemID == 0)
                    apibridge.From.SystemID = SolarSystem.GetIDByName(apibridge.From.System);

                if (apibridge.To.SystemID == 0)
                    apibridge.To.SystemID = SolarSystem.GetIDByName(apibridge.To.System);

                JumpBridge bridge = new JumpBridge
                                        {
                                            From = SolarSystem.GetByID(apibridge.From.SystemID),
                                            To = SolarSystem.GetByID(apibridge.To.SystemID),
                                            Base = new JumpBridgePos
                                                       {
                                                           Planet = apibridge.From.Planet,
                                                           Moon = apibridge.From.Moon,
                                                       },
                                            Destination = new JumpBridgePos
                                                              {
                                                                  Planet = apibridge.To.Planet,
                                                                  Moon = apibridge.To.Moon,
                                                              },
                                            BridgeLine = Color.FromName(apibridge.Color),
                                            Corporation = apibridge.Corporation,
                                            Password = apibridge.Password,
                                        };


                DynamicStore.Instance.JumpBridges.Add(bridge);
            }

            UpdateListView();

            Cursor = regularCursor;
        }
    }
}