﻿using System;
using System.Collections.Generic;
using System.Diagnostics;
using System.Windows.Forms;
using EveMapSettings;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.EveAPI.Data;

namespace EVEMapNG.SubForms
{
    public partial class Settings : Form
    {
        public Settings()
        {
            InitializeComponent();

            chkRegionNames.Checked = UserSettings.Instance.ActiveViewportStyle.ShowRegionNames;
            chkConstellationNames.Checked = UserSettings.Instance.ActiveViewportStyle.ShowConstNames;
            chkSystemNames.Checked = UserSettings.Instance.ActiveViewportStyle.ShowSystemNames;
            chkDetailText.Checked = UserSettings.Instance.ActiveViewportStyle.ShowSystemDetails;
            chkJumpBridges.Checked = UserSettings.Instance.ActiveViewportStyle.ShowJumpbridges;
            chkCynoBeacons.Checked = UserSettings.Instance.ActiveViewportStyle.ShowCynoGenerators;
            chkSovCircles.Checked = UserSettings.Instance.ActiveViewportStyle.ShowSovCircles;
            chkMiniMap.Checked = UserSettings.Instance.ActiveViewportStyle.ShowMiniMap;

            pnlBackground.BackColor = UserSettings.Instance.ActiveColorScheme.BackgroundColor;
            pnlSystem.BackColor = UserSettings.Instance.ActiveColorScheme.SystemTextColor;
            pnlCynoBeacon.BackColor = UserSettings.Instance.ActiveColorScheme.CynoBeaconColor;
            pnlNormalGate.BackColor = UserSettings.Instance.ActiveColorScheme.GateGenericColor;
            pnlConstellationGate.BackColor = UserSettings.Instance.ActiveColorScheme.GateConstellationColor;
            pnlRegionGate.BackColor = UserSettings.Instance.ActiveColorScheme.GateRegionColor;
            pnlGateRoute.BackColor = UserSettings.Instance.ActiveColorScheme.GateRouteColor;
            pnlJumpBridge.BackColor = UserSettings.Instance.ActiveColorScheme.JumpRouteColor;

            pnlMinimapBackground.BackColor = UserSettings.Instance.ActiveColorScheme.MinimapBackgroundColor;
            pnlMinimapBorder.BackColor = UserSettings.Instance.ActiveColorScheme.MinimapBorderColor;
            pnlMinimapFocus.BackColor = UserSettings.Instance.ActiveColorScheme.MinimapFocusBackgroundColor;
            pnlMinimapFocusOutline.BackColor = UserSettings.Instance.ActiveColorScheme.MinimapFocusOutlineColor;

            routeWeightSliderGateBridge.Initialize("Bridges", EdgeWeights.GateBridge);
            routeWeightSliderGateDefault.Initialize("Default", EdgeWeights.GateDefault);
            routeWeightSliderGateHigh.Initialize("Avoids", EdgeWeights.GateHigh);
            routeWeightSliderJumpBridge.Initialize("Bridges", EdgeWeights.JumpBridge);
            routeWeightSliderJumpDefault.Initialize("Default", EdgeWeights.JumpDefault);
            routeWeightSliderJumpBeacon.Initialize("Beacons", EdgeWeights.JumpBeacon);
            routeWeightSliderJumpHigh.Initialize("Avoids", EdgeWeights.JumpHigh);
            routeWeightSliderJumpStation.Initialize("Station Systems", EdgeWeights.JumpStation);

            txtRestJumpbridges.Text = UserSettings.Instance.RestJumpbridgeUrl;
            txtRestCynos.Text = UserSettings.Instance.RestCynoBeaconUrl;

            //Init EVE API tab
            if (DynamicStore.Instance.ApiData.ApiInfo.FullApiKey != null)
            {
                tbApiKey.Text = DynamicStore.Instance.ApiData.ApiInfo.FullApiKey;
                tbUserId.Text = DynamicStore.Instance.ApiData.ApiInfo.UserID.ToString();
            }

            if (Character.SelectedCharacter != null)
                tbCharacter.Text = Character.SelectedCharacter.Name;
        }

        private void Panel_Click(object sender, EventArgs e)
        {
            cldColorPicker.ShowDialog();
            ((Panel) sender).BackColor = cldColorPicker.Color;
        }

        private void btnSave_Click(object sender, EventArgs e)
        {
            //Copy form values to UserSettings.Instance.
            UserSettings.Instance.ActiveViewportStyle.ShowRegionNames = chkRegionNames.Checked;
            UserSettings.Instance.ActiveViewportStyle.ShowConstNames = chkConstellationNames.Checked;
            UserSettings.Instance.ActiveViewportStyle.ShowSystemNames = chkSystemNames.Checked;
            UserSettings.Instance.ActiveViewportStyle.ShowSystemDetails = chkDetailText.Checked;
            UserSettings.Instance.ActiveViewportStyle.ShowJumpbridges = chkJumpBridges.Checked;
            UserSettings.Instance.ActiveViewportStyle.ShowCynoGenerators = chkCynoBeacons.Checked;
            UserSettings.Instance.ActiveViewportStyle.ShowSovCircles = chkSovCircles.Checked;
            UserSettings.Instance.ActiveViewportStyle.ShowMiniMap = chkMiniMap.Checked;

            // save user color scheme
            UserSettings.Instance.ActiveColorScheme.BackgroundColor = pnlBackground.BackColor;
            UserSettings.Instance.ActiveColorScheme.SystemTextColor = pnlSystem.BackColor;
            UserSettings.Instance.ActiveColorScheme.CynoBeaconColor = pnlCynoBeacon.BackColor;
            UserSettings.Instance.ActiveColorScheme.GateGenericColor = pnlNormalGate.BackColor;
            UserSettings.Instance.ActiveColorScheme.GateConstellationColor = pnlConstellationGate.BackColor;
            UserSettings.Instance.ActiveColorScheme.GateRegionColor = pnlRegionGate.BackColor;
            UserSettings.Instance.ActiveColorScheme.GateRouteColor = pnlGateRoute.BackColor;
            UserSettings.Instance.ActiveColorScheme.JumpRouteColor = pnlJumpBridge.BackColor;

            //save user minimap color scheme
            UserSettings.Instance.ActiveColorScheme.MinimapBackgroundColor = pnlMinimapBackground.BackColor;
            UserSettings.Instance.ActiveColorScheme.MinimapBorderColor = pnlMinimapBorder.BackColor;
            UserSettings.Instance.ActiveColorScheme.MinimapFocusBackgroundColor = pnlMinimapFocus.BackColor;
            UserSettings.Instance.ActiveColorScheme.MinimapFocusOutlineColor = pnlMinimapFocusOutline.BackColor;

            UserSettings.Instance.RestJumpbridgeUrl = txtRestJumpbridges.Text;
            UserSettings.Instance.RestCynoBeaconUrl = txtRestCynos.Text;

            // save user's preferences
            //BUG:  REDO THIS

            //Copy slider values to route data
            EdgeWeights.SetGateWeights(routeWeightSliderGateHigh.trackBar1.Value,
                                       routeWeightSliderGateBridge.trackBar1.Value,
                                       routeWeightSliderGateDefault.trackBar1.Value);

            EdgeWeights.SetJumpWeights(routeWeightSliderJumpHigh.trackBar1.Value,
                                       routeWeightSliderJumpDefault.trackBar1.Value,
                                       routeWeightSliderJumpBridge.trackBar1.Value,
                                       routeWeightSliderJumpBeacon.trackBar1.Value,
                                       routeWeightSliderJumpStation.trackBar1.Value);

            // save user's preferences
            //BUG: REDO THIS


            //Handle EVE API Stuff
            if (NewCharacterList.Count > 0)
            {
                foreach (Character character in NewCharacterList)
                {
                    if (!DynamicStore.Instance.ApiData.Characters.ContainsKey(character.CharacterId))
                        DynamicStore.Instance.ApiData.Characters.Add(character.CharacterId, character);
                }
                DynamicStore.Instance.ApiData.ApiInfo.FullApiKey = tbApiKey.Text;
                DynamicStore.Instance.ApiData.ApiInfo.UserID = Int32.Parse(tbUserId.Text);
            }
            Close();
        }

        private void btnCancel_Click(object sender, EventArgs e)
        {
            Close();
        }

        #region #EVEAPI

        private List<Character> NewCharacterList = new List<Character>();

        private void btnGetCharacters_Click(object sender, EventArgs e)
        {
            if (tbUserId.Text == string.Empty || tbApiKey.Text == string.Empty)
            {
                MessageBox.Show("Please enter your userId and ApiKey");
                return;
            }

            NewCharacterList = Character.GetCharacters(tbApiKey.Text, Int32.Parse(tbUserId.Text));
            CharacterSelect characterSelect = new CharacterSelect(NewCharacterList);

            if (characterSelect.ShowDialog() == DialogResult.OK)
            {
                tbCharacter.Text = characterSelect.selectedCharacter.Name;
                characterSelect.selectedCharacter.Selected = true;
                foreach (Character c in NewCharacterList)
                {
                    if (c.Name == characterSelect.selectedCharacter.Name)
                        c.Selected = true;
                    else
                        c.Selected = false;
                }
            }
        }

        private void linkLabel1_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Process.Start("http://myeve.eve-online.com/api/default.asp");
        }

        #endregion
    }
}