#region

using System;
using System.Windows.Forms;
using libEveMapNG.Exceptions;
using libEveMapNG.Utility;

#endregion

namespace EVEMapNG.SubForms
{
    public partial class UnhandledExceptionWindow : Form
    {
        private readonly Exception _exception;

        public UnhandledExceptionWindow()
        {
            InitializeComponent();
        }

        public UnhandledExceptionWindow(Exception err)
            : this()
        {
            _exception = err;
        }

        private void UnhandledExceptionWindow_Load(object sender, EventArgs e)
        {
            try
            {
                OperatingSystem os = Environment.OSVersion;
                string exceptionReport = "EVE-Map NG Version: " + Application.ProductVersion;
                exceptionReport += Environment.NewLine;
                exceptionReport += ".NET Runtime Version: " + Environment.Version;
                exceptionReport += Environment.NewLine;
                exceptionReport += "Operating System: " + os.VersionString;
                exceptionReport += Environment.NewLine;
                exceptionReport += Environment.NewLine;
                exceptionReport += _exception.ToString();
                textBox1.Text = exceptionReport;
            }
            catch (Exception ex)
            {
                ExceptionHandler.LogException(ex, true);
                textBox1.Text = "Error retrieving error data. Wow, things are *really* screwed up!";
            }
        }

        private void btnClose_Click(object sender, EventArgs e)
        {
            DialogResult = DialogResult.OK;
            Close();
        }

        private void llCopy_LinkClicked(object sender, LinkLabelLinkClickedEventArgs e)
        {
            Clipboard.SetText(textBox1.Text, TextDataFormat.Text);
            MessageBox.Show("The error details have been copied to the clipboard.", "Copy", MessageBoxButtons.OK,
                            MessageBoxIcon.Information);
        }

        private void btnReset_Click(object sender, EventArgs e)
        {
            DialogResult dr = MessageBox.Show(
                "This will clear all your saved settings.  You should only try this if EVE-Map NG has errored more " +
                "than once.\r\n\r\nClear settings?",
                "Clear Settings?",
                MessageBoxButtons.YesNo,
                MessageBoxIcon.Warning,
                MessageBoxDefaultButton.Button2);


            if (dr == DialogResult.Yes)
            {
                ResetSettings();
                MessageBox.Show("Your settings have been reset.",
                                "Settings Reset", MessageBoxButtons.OK, MessageBoxIcon.Information);
                DialogResult = DialogResult.OK;
                Application.Exit();
            }
        }

        private static void ResetSettings()
        {
            Files.DeleteAllFiles();
        }
    }
}