﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using EVEMapNG.SubForms;
using EveMapSettings;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;
using libEveMapNG.Routes;
using libEveMapNG.Types;

namespace EveMapControls.MainFormTabs
{
    public partial class GatePlanner : UserControl
    {
        private List<Vertex> _currentGateRoute;

        public GatePlanner()
        {
            InitializeComponent();
        }

        public MapViewport Viewport { get; set; }

        public List<Vertex> CurrentRoute
        {
            get { return _currentGateRoute; }
        }

        public void RestoreState()
        {
            if (UserSettings.Instance.AppState.GPState == null)
                UserSettings.Instance.AppState.LoadDefaultSettings();

            rbAvoidHigh.Checked = UserSettings.Instance.AppState.GPState.AvoidHigh;
            rbAvoidLow.Checked = UserSettings.Instance.AppState.GPState.AvoidLow;
            rbAvoidNone.Checked = UserSettings.Instance.AppState.GPState.AvoidNone;
            rbAvoidNull.Checked = UserSettings.Instance.AppState.GPState.AvoidNull;

            if (UserSettings.Instance.AppState.GPState.GateDestinationSystemId != 0)
                ssbGateDestinationSystem.SelectedSystem =
                    SolarSystem.GetByID(UserSettings.Instance.AppState.GPState.GateDestinationSystemId);

            if (UserSettings.Instance.AppState.GPState.GateStartingSystemId != 0)
                ssbGateStartingSystem.SelectedSystem =
                    SolarSystem.GetByID(UserSettings.Instance.AppState.GPState.GateStartingSystemId);

            chkGateFilterRoute.Checked = UserSettings.Instance.AppState.GPState.FilterRoute;
            txtShipMass.Text = UserSettings.Instance.AppState.GPState.ShipMass;
            lblShipName.Text = UserSettings.Instance.AppState.GPState.ShipName;
            chkGateUseBridges.Checked = UserSettings.Instance.AppState.GPState.UseBridges;
        }

        public void SaveState()
        {
            UserSettings.Instance.AppState.GPState.AvoidHigh = rbAvoidHigh.Checked;
            UserSettings.Instance.AppState.GPState.AvoidLow = rbAvoidLow.Checked;
            UserSettings.Instance.AppState.GPState.AvoidNone = rbAvoidNone.Checked;
            UserSettings.Instance.AppState.GPState.AvoidNull = rbAvoidNull.Checked;

            if (ssbGateDestinationSystem.SelectedSystem != null)
                UserSettings.Instance.AppState.GPState.GateDestinationSystemId =
                    ssbGateDestinationSystem.SelectedSystem.ID;

            if (ssbGateStartingSystem.SelectedSystem != null)
                UserSettings.Instance.AppState.GPState.GateStartingSystemId = ssbGateStartingSystem.SelectedSystem.ID;

            UserSettings.Instance.AppState.GPState.FilterRoute = chkGateFilterRoute.Checked;
            UserSettings.Instance.AppState.GPState.ShipMass = txtShipMass.Text;
            UserSettings.Instance.AppState.GPState.ShipName = lblShipName.Text;
            UserSettings.Instance.AppState.GPState.UseBridges = chkGateUseBridges.Checked;
        }

        private void lvGateRoute_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lvGateRoute.SelectedItems.Count > 0 && lvGateRoute.SelectedItems[0].Tag != null)
                Viewport.CenterOnPoint(((SolarSystem) lvGateRoute.SelectedItems[0].Tag).OrganicCoordinates);
        }

        private void btnSelectedSystem_Click(object sender, EventArgs e)
        {
            switch (((Button) sender).Name)
            {
                case ("btnSelectGateStart"):
                    ssbGateStartingSystem.Text = Viewport.SelectedSystem.Name;
                    break;
                case ("btnSelectGateEnd"):
                    ssbGateDestinationSystem.Text = Viewport.SelectedSystem.Name;
                    break;
                default:
                    break;
            }
        }

        private void chkFilterGateRoutes_CheckedChanged(object sender, EventArgs e)
        {
            Viewport.FilterSystems = chkGateFilterRoute.Checked;
            if (chkGateFilterRoute.Checked && _currentGateRoute != null)
            {
                Dictionary<int, SolarSystem> unfilteredSystems =
                    _currentGateRoute.ToDictionary(route => route.SolarSystem.ID, route => route.SolarSystem);
                Viewport.UnfilteredSystems = unfilteredSystems;
            }
            Viewport.Invalidate();
        }

        public void SetRouteStart(SolarSystem system)
        {
            ssbGateStartingSystem.SelectedSystem = system;
            ssbGateStartingSystem.Text = system.Name;
        }

        public void SetRouteEnd(SolarSystem system)
        {
            ssbGateDestinationSystem.SelectedSystem = system;
            ssbGateDestinationSystem.Text = system.Name;
        }

        public void AvoidSystem(SolarSystem system)
        {
            slGateAvoid.Add(system);
        }

        public void AddWaypoint(SolarSystem system)
        {
            slGateWaypoints.Add(system);
        }

        private void btnClear_Click(object sender, EventArgs e)
        {
            ssbGateStartingSystem.Text = string.Empty;
            ssbGateDestinationSystem.Text = string.Empty;
            Viewport.Route = null;
            lvGateRoute.Items.Clear();
            slGateAvoid.Clear();
            slGateWaypoints.Clear();
            Viewport.Invalidate();
        }

        private void btnFindRoute_Click(object sender, EventArgs e)
        {
            if (SolarSystem.GetByName(ssbGateStartingSystem.Text) == null ||
                SolarSystem.GetByName(ssbGateDestinationSystem.Text) == null)
            {
                MessageBox.Show(
                    "You must select a valid starting system and destination system to plot a route.",
                    "USER FAIL");
                return;
            }

            float customMass;
            if (txtShipMass.Text != "")
                float.TryParse(txtShipMass.Text, out customMass);
            else
                customMass = 1f; //If theres no mass picked (or the parse fails) leave it at 1
            GateRouteParameters myRoute = new GateRouteParameters
                                              {
                                                  StartSystem = SolarSystem.GetByName(ssbGateStartingSystem.Text),
                                                  DestSystem = SolarSystem.GetByName(ssbGateDestinationSystem.Text),
                                                  RoutingOption = GetRoutingOption(),
                                                  AvoidSystems = slGateAvoid.SelectedSystems,
                                                  WayPoints = slGateWaypoints.SelectedSystems,
                                                  UseBridges = chkGateUseBridges.Checked,
                                                  ShipMass = customMass
                                              };

            GateRouter gateRoute = new GateRouter();
            List<Vertex> route = gateRoute.GetRoute(myRoute);
            lvGateRoute.Items.Clear();

            int jumpCount = 1;
            int ozoneUsed = 0;

            /* Loop the route list and build the listview items */
            for (int i = 0; i < route.Count - 1; i++)
            {
                SolarSystem current = route[i].SolarSystem;
                SolarSystem next = route[i + 1].SolarSystem;

                ListViewItem myItem = new ListViewItem(current.Name) {Tag = current};
                if (!Equals(null, route[i + 1].Bridge.Pos))
                {
                    myItem.SubItems.Add(route[i + 1].Bridge.Pos.Planet + "-" + route[i + 1].Bridge.Pos.Moon);
                    myItem.SubItems.Add(next.Name);
                    myItem.SubItems.Add(jumpCount.ToString());
                    myItem.SubItems.Add(route[i + 1].FuelCost.ToString());

                    ozoneUsed += route[i + 1].FuelCost;
                }
                else
                {
                    myItem.SubItems.Add("");
                    myItem.SubItems.Add(next.Name);
                    myItem.SubItems.Add(jumpCount.ToString());
                }

                jumpCount++;
                lvGateRoute.Items.Add(myItem);
            }

            ListViewItem summary = new ListViewItem("Total Ozone: ");
            summary.SubItems.AddRange(new[] {"", "", ""}); //whitespace padding
            summary.SubItems.Add(ozoneUsed.ToString());
            lvGateRoute.Items.Add(summary);

            _currentGateRoute = route;
            Viewport.Route = route;
            Viewport.Invalidate();
        }

        private GateRouteOption GetRoutingOption()
        {
            if (rbAvoidHigh.Checked)
                return GateRouteOption.AvoidHighSec;
            if (rbAvoidLow.Checked)
                return GateRouteOption.AvoidLowSec;
            if (rbAvoidNull.Checked)
                return GateRouteOption.AvoidNullSec;
            //Default option to use whatever.
            return GateRouteOption.None;
        }

        private void btnPickShip_Click(object sender, EventArgs e)
        {
            PickShipType pickShipTypeForm = new PickShipType();
            if (pickShipTypeForm.ShowDialog() == DialogResult.OK)
            {
                lblShipName.Text = String.Format("Ship: {0}", pickShipTypeForm.ShipName);
                txtShipMass.Text = pickShipTypeForm.Mass.ToString();
            }
            if (_currentGateRoute != null)
                btnFindRoute_Click(null, null);
        }

        private void btnGateClipboard_Click(object sender, EventArgs e)
        {
            Clipboard.Clear();
            StringBuilder sb = new StringBuilder();
            int ozoneNeeded = 0;
            for (int i = 0; i < _currentGateRoute.Count - 1; i++)
            {
                Vertex item = _currentGateRoute[i];
                sb.Append(String.Format("{0}, ", item.SolarSystem.Name));
                ozoneNeeded += item.FuelCost;
            }
            sb.Append(String.Format("{0} - Liquid Ozone Needed: {1} Total Jumps: {2}",
                                    _currentGateRoute[_currentGateRoute.Count - 1].SolarSystem.Name, ozoneNeeded,
                                    _currentGateRoute.Count));
            Clipboard.SetText(sb.ToString());
        }
    }
}