﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Windows.Forms;
using EveMapSettings;
using libEveMapNG.Data.Dynamic;
using libEveMapNG.Data.Static;
using libEveMapNG.Routes;
using libEveMapNG.Types;
using libEveMapNG.Utility;

namespace EveMapControls.MainFormTabs
{
    public partial class JumpPlanner : UserControl
    {
        private const int JUMPTARGETSYSTEMS = 4219; // This magic number is the number of 0.0 and lowsec systems in eve.
        private List<Vertex> CurrentJumpRoute;
        private JumpRouteParameters CurrentJumpRouteParameters;

        public JumpPlanner()
        {
            InitializeComponent();
            cboShip.Items.RemoveAt(0);
            foreach (KeyValuePair<string, CapitalShips.Ship> ship in CapitalShips.Ships)
                cboShip.Items.Add(ship.Value.Name);
        }

        public MapViewport Viewport { get; set; }
        public RouteWebServer WebServer { get; set; }

        public void RestoreState()
        {
            if (UserSettings.Instance.AppState.JPState == null)
                UserSettings.Instance.AppState.LoadDefaultSettings();

            if (UserSettings.Instance.AppState.JPState.JumpDestinationSystemId != 0)
                ssbJumpDestinationSystem.SelectedSystem =
                    SolarSystem.GetByID(UserSettings.Instance.AppState.JPState.JumpDestinationSystemId);

            cboJDC.SelectedIndex = UserSettings.Instance.AppState.JPState.JDC;
            cboJFC.SelectedIndex = UserSettings.Instance.AppState.JPState.JFC;
            cboJF.SelectedIndex = UserSettings.Instance.AppState.JPState.JF;
            cboShip.SelectedIndex = UserSettings.Instance.AppState.JPState.Ship;

            if (UserSettings.Instance.AppState.JPState.JumpDestinationSystemId != 0)
                ssbJumpStartingSystem.SelectedSystem =
                    SolarSystem.GetByID(UserSettings.Instance.AppState.JPState.JumpStartingSystemId);

            chkJumpBeacons.Checked = UserSettings.Instance.AppState.JPState.UseBeacons;
            //chkJumpBridges.Checked = UserSettings.Instance.AppState.UseJumpBridges;
        }

        public void SaveState()
        {
            if (ssbJumpDestinationSystem.SelectedSystem != null)
                UserSettings.Instance.AppState.JPState.JumpDestinationSystemId =
                    ssbJumpDestinationSystem.SelectedSystem.ID;

            UserSettings.Instance.AppState.JPState.JDC = cboJDC.SelectedIndex;
            UserSettings.Instance.AppState.JPState.JFC = cboJFC.SelectedIndex;
            UserSettings.Instance.AppState.JPState.JF = cboJF.SelectedIndex;
            UserSettings.Instance.AppState.JPState.Ship = cboShip.SelectedIndex;

            if (ssbJumpStartingSystem.SelectedSystem != null)
                UserSettings.Instance.AppState.JPState.JumpStartingSystemId = ssbJumpStartingSystem.SelectedSystem.ID;

            UserSettings.Instance.AppState.JPState.UseBeacons = chkJumpBeacons.Checked;
            //UserSettings.Instance.AppState.UseJumpBridges = chkJumpBridges.Checked;
        }

        private void lvRoute_SelectedIndexChanged(object sender, EventArgs e)
        {
            if (lvRoute.SelectedItems.Count > 0 && lvRoute.SelectedItems[0].Tag != null)
            {
                Viewport.CenterOnPoint(
                    ((Vertex) (lvRoute.SelectedItems[0].Tag)).SolarSystem.OrganicCoordinates);
            }
        }

        //Load up a parameter structure, and then call the background worker to work off of those parameters.
        private void btnJumpFindRoute_Click(object sender, EventArgs e)
        {
            lvRoute.Items.Clear();
            if (SolarSystem.GetByName(ssbJumpStartingSystem.Text) == null ||
                SolarSystem.GetByName(ssbJumpDestinationSystem.Text) == null ||
                (string) cboShip.SelectedItem == "")
            {
                MessageBox.Show(
                    "You must select a valid starting system, destination system, have jump skills trained, and a ship to plot a route.",
                    "USER FAIL");
                return;
            }

            foreach (SolarSystem waypoint in slJumpWaypoints.SelectedSystems)
            {
                if (waypoint.Security >= 0.45)
                {
                    MessageBox.Show("Cannot plot cyno route with waypoints in hisec", "You're retarded");
                    btnJumpFindRoute.Enabled = true;
                    return;
                }
            }


            btnJumpFindRoute.Enabled = false;

            pbMain.Value = 0;

            if (slJumpWaypoints.SelectedSystems.Count > 0)
                pbMain.Maximum = JUMPTARGETSYSTEMS*(1 + slJumpWaypoints.SelectedSystems.Count);
            else
                pbMain.Maximum = JUMPTARGETSYSTEMS;

            SolarSystem startSystem = SolarSystem.GetByName(ssbJumpStartingSystem.Text);
            SolarSystem endSystem = SolarSystem.GetByName(ssbJumpDestinationSystem.Text);
            if (endSystem.Security >= 0.45)
            {
                MessageBox.Show("Cannot plot cyno route into hisec.", "Invalid Destination");
                btnJumpFindRoute.Enabled = true;
                return;
            }

            int jdc, jf, jfc;
            if (!int.TryParse((String) cboJDC.SelectedItem, out jdc) ||
                !int.TryParse((String) cboJF.SelectedItem, out jf) ||
                !int.TryParse((String) cboJFC.SelectedItem, out jfc))
            {
                MessageBox.Show("Please select valid JDC/JF/JFC Values.", "Invalid Parameters");
                btnJumpFindRoute.Enabled = true;
                return;
            }

            CapitalShips.Ship ship;
            if (!CapitalShips.Ships.TryGetValue(cboShip.Text, out ship))
            {
                MessageBox.Show("Please select a valid ship.", "Invalid Ship");
                btnJumpFindRoute.Enabled = true;
                return;
            }

            JumpRouteParameters jumpRoute = new JumpRouteParameters
                                                {
                                                    StartSystem = startSystem,
                                                    DestSystem = endSystem,
                                                    CapitalShipProfile =
                                                        new ShipProfile
                                                            {
                                                                JDC = jdc,
                                                                Ship = ship,
                                                                JF = jf,
                                                                JFC = jfc
                                                            },
                                                    UseBeacons = chkJumpBeacons.Checked,
                                                    //UseBridges = chkJumpBridges.Checked,
                                                    WayPoints = slJumpWaypoints.SelectedSystems
                                                };
            foreach (SolarSystem system in slJumpAvoid.SelectedSystems)
                jumpRoute.AvoidSystems.Add(system);

            CurrentJumpRouteParameters = jumpRoute;
            pbMain.Visible = true;
            bgwCyno.RunWorkerAsync();
        }

        //Slow calculation runs in the background
        private void CynoBackGroundWorker_DoWork(object sender, DoWorkEventArgs e)
        {
            JumpRouter CynoRouteFinder = new JumpRouter();
            CynoRouteFinder.SystemProcessed += CynoBackGroundWorker_ProgressChanged;
            List<Vertex> optimalRoute = CynoRouteFinder.GetRoute(CurrentJumpRouteParameters);
            CurrentJumpRoute = optimalRoute;
            //WebServer.JumpRoute = optimalRoute;
        }

        //Once the Cynoroute background worker thread is done, print out the details to the listview
        private void CynoBackGroundWorker_RunWorkerCompleted(object sender, RunWorkerCompletedEventArgs e)
        {
            int TotalFuel = 0;
            float TotalLightYears = 0;


            for (int i = 0; i < CurrentJumpRoute.Count - 1; i++)
            {
                SolarSystem CurrentSystem = CurrentJumpRoute[i].SolarSystem;
                SolarSystem NextSystem = CurrentJumpRoute[i + 1].SolarSystem;
                Vertex CurrentVertex = CurrentJumpRoute[i];
                Vertex NextVertex = CurrentJumpRoute[i + 1];

                ListViewItem myItem = new ListViewItem(CurrentSystem.Name) {Tag = CurrentJumpRoute[i]};
                myItem.SubItems.Add(NextSystem.Name);
                myItem.SubItems.Add(JumpDrive.GetDistance(CurrentSystem, NextSystem).ToString());
                myItem.SubItems.Add(NextVertex.FuelCost.ToString());
                myItem.SubItems.Add(NextVertex.JumpType.ToString());

                lvRoute.Items.Add(myItem);

                TotalFuel += NextVertex.FuelCost;
                TotalLightYears += JumpDrive.GetDistance(CurrentSystem, NextSystem);
            }


            ListViewItem summaryRow = new ListViewItem("Totals:");
            summaryRow.SubItems.Add("");
            summaryRow.SubItems.Add(TotalLightYears.ToString());
            summaryRow.SubItems.Add(TotalFuel.ToString());
            summaryRow.SubItems.Add((CurrentJumpRoute.Count - 1).ToString());
            lvRoute.Items.Add(summaryRow);

            pbMain.Visible = false;
            Viewport.Route = CurrentJumpRoute;
            Viewport.FilterSystems = true;
            Viewport.FilterType = MapViewport.SystemFilterType.Route;
            Viewport.Invalidate();
            btnJumpFindRoute.Enabled = true;
        }

        private void PbIncrement()
        {
            pbMain.Increment(1);
        }

        private void CynoBackGroundWorker_ProgressChanged(object sender, ProgressChangedEventArgs e)
        {
            IncrementProgressBar a = PbIncrement;
            pbMain.BeginInvoke(a);
        }

        private void btnCopyJumpClipboard_Click(object sender, EventArgs e)
        {
            Clipboard.Clear();
            SolarSystem prevSys = SolarSystem.GetByName(ssbJumpStartingSystem.Text);
            int totalFuel = 0;
            string RouteText = string.Format("{0} - {1}/{2} Route: ",
                                             cboShip.Text,
                                             cboJDC.Text,
                                             cboJFC.Text);
            if (!Equals(CurrentJumpRoute, null))
            {
                foreach (Vertex SS in CurrentJumpRoute)
                {
                    SolarSystem system = SS.SolarSystem;
                    if (SS == CurrentJumpRoute[CurrentJumpRoute.Count - 1])
                        RouteText += string.Format("{0}\r\n", system.Name);
                    else
                        RouteText += string.Format("{0} -> ", system.Name);
                    totalFuel += JumpDrive.GetFuel(prevSys, system, CurrentJumpRouteParameters);
                    prevSys = system;
                }
                RouteText += string.Format(" {0} Jumps, Fuel: {1}  Round Trip: {2} ", CurrentJumpRoute.Count - 1,
                                           totalFuel,
                                           totalFuel*2);
                Clipboard.SetText(RouteText);
            }
        }

        private void btnClearJump_Click(object sender, EventArgs e)
        {
            ssbJumpStartingSystem.Text = string.Empty;
            ssbJumpStartingSystem.SelectedItem = null;
            ssbJumpDestinationSystem.Text = string.Empty;
            ssbJumpDestinationSystem.SelectedItem = null;
            lvRoute.Items.Clear();
            slJumpWaypoints.Clear();
            slJumpAvoid.Clear();
            Viewport.Route = null;
            Viewport.Invalidate();
        }

        public void SetJumpStart(SolarSystem system)
        {
            ssbJumpStartingSystem.SelectedSystem = system;
        }

        public void SetJumpEnd(SolarSystem system)
        {
            ssbJumpDestinationSystem.SelectedSystem = system;
        }

        public void AddJumpAvoid(SolarSystem system)
        {
            slJumpAvoid.Add(system);
        }

        public void AddJumpWaypoint(SolarSystem system)
        {
            slJumpWaypoints.Add(system);
        }

        private void btnSelectJumpStart_Click(object sender, EventArgs e)
        {
            ssbJumpStartingSystem.SelectedSystem = Viewport.SelectedSystem;
        }

        private void btnSelectJumpEnd_Click(object sender, EventArgs e)
        {
            ssbJumpDestinationSystem.SelectedSystem = Viewport.SelectedSystem;
        }

        #region Nested type: IncrementProgressBar

        private delegate void IncrementProgressBar();

        #endregion
    }
}