﻿using System;
using System.Collections.Generic;
using libEveMapNG.Utility;

namespace EveMapSettings
{
    [Serializable]
    public static class Singleton
    {
        private static UserSettings instance;

        public static UserSettings Instance
        {
            get { return instance; }
            set { instance = value; }
        }
    }

    [Serializable]
    public class UserSettings
    {
        private readonly Dictionary<string, MapColorScheme> UserMapColorSchemes =
            new Dictionary<string, MapColorScheme>();

        private readonly Dictionary<string, ViewportSettings> UserViewportStyles =
            new Dictionary<string, ViewportSettings>();

        public ApplicationState AppState;
        public bool FirstRun = true;

        public string RestCynoBeaconUrl;
        public string RestJumpbridgeUrl;
        private string _activeMapColorScheme;
        private string _activeViewportStyle;

        public static UserSettings Instance
        {
            get { return Singleton.Instance; }
            set { Singleton.Instance = value; }
        }

        public ViewportSettings ActiveViewportStyle
        {
            get { return UserViewportStyles[_activeViewportStyle]; }
        }

        public MapColorScheme ActiveColorScheme
        {
            get { return UserMapColorSchemes[_activeMapColorScheme]; }
        }

        public static void Save()
        {
            Serializer.WriteToAppData("settings.store", Instance);
        }

        public static void Read()
        {
            Instance = Serializer.ReadFromAppData<UserSettings>("settings.store");
        }

        public void AddUserViewportStyle(ViewportSettings settings)
        {
            if (UserViewportStyles.ContainsKey(settings.Name))
                return;
            UserViewportStyles.Add(settings.Name, settings);
            _activeViewportStyle = settings.Name;
        }

        public void RemoveUserViewportStyle(ViewportSettings settings)
        {
            if (UserViewportStyles.ContainsKey(settings.Name))
                UserViewportStyles.Remove(settings.Name);
        }

        public void SetActiveViewportStyle(string name)
        {
            _activeViewportStyle = name;
        }

        public void AddMapColorScheme(MapColorScheme scheme)
        {
            if (UserMapColorSchemes.ContainsKey(scheme.Name))
                return;
            UserMapColorSchemes.Add(scheme.Name, scheme);
            _activeMapColorScheme = scheme.Name;
        }

        public void RemoverUserMapColorScheme(MapColorScheme scheme)
        {
            if (UserMapColorSchemes.ContainsKey(scheme.Name))
                UserMapColorSchemes.Remove(scheme.Name);
        }

        public void SetActiveColorScheme(string name)
        {
            _activeMapColorScheme = name;
        }
    }
}