#region

using System;
using System.Collections.Generic;

#endregion

namespace Eve_Class_Library
{
    public class EVEMinerals
    {
        #region IceTypes enum

        [Flags]
        public enum IceTypes
        {
            None = 0x000,
            BlueIce = 0x001, /* gallente highsec */
            ClearIcicle = 0x002, /* amarr highsec */
            WhiteGlaze = 0x004, /* caldari highsec  */
            GlacialMass = 0x008, /* minmatar highsec */
            EnrichedClearIcicle = 0x010, /* blood raider, sansha */
            PristineWhiteGlaze = 0x020, /* guristas */
            SmoothGlacialMass = 0x040, /* angels */
            ThickBlueIce = 0x080, /* serpentis */
            GlareCrust = 0x100, /* > -0.15, < 0.35 */
            DarkGlitter = 0x200, /* > -0.45, < 0.15 */
            Gelidus = 0x400, /* > -0.65, < -0.15 */
            Krystallos = 0x800 /* < -0.45 */
        }

        #endregion

        #region OreTypes enum

        [Flags]
        public enum OreTypes
        {
            None = 0x0000,
            Veldspar = 0x0001,
            Scordite = 0x0002,
            Pyroxeres = 0x0004,
            Plagioclase = 0x0008,
            Jaspet = 0x0010,
            Gneiss = 0x0020,
            Kernite = 0x0040,
            Omber = 0x0080,
            Spodumain = 0x0100,
            Hemorphite = 0x0200,
            Hedbergite = 0x0400,
            DarkOchre = 0x0800,
            Crokite = 0x1000,
            Bistot = 0x2000,
            Mercoxit = 0x4000,
            Arkonor = 0x8000
        }

        #endregion

        public static OreTypes GetOres(EVESystem system)
        {
            OreTypes Compound = OreTypes.None;

            switch (system.SecurityClassLetter)
            {
                case 0: // A
                    Compound |= OreTypes.Veldspar | OreTypes.Scordite;
                    break;
                case 1: // B
                    Compound |= OreTypes.Veldspar | OreTypes.Scordite;
                    switch (system.SecurityClassNumber)
                    {
                        case 3:
                            Compound |= OreTypes.Hemorphite;
                            goto case 2;
                        case 2:
                            Compound |= OreTypes.Jaspet;
                            goto case 1;
                        case 1:
                            Compound |= OreTypes.Kernite;
                            goto case 0;
                        case 0:
                            if (system.ParentRegion.ParentFaction != null &&
                                system.ParentRegion.ParentFaction.Name == "Gallente Federation")
                                Compound |= OreTypes.Plagioclase;
                            else Compound |= OreTypes.Pyroxeres;
                            break;
                    }
                    break;
                case 2: // C
                    Compound |= OreTypes.Veldspar | OreTypes.Scordite | OreTypes.Plagioclase | OreTypes.Pyroxeres;
                    switch (system.SecurityClassNumber)
                    {
                        case 2:
                            Compound |= OreTypes.Hedbergite;
                            goto case 1;
                        case 1:
                            Compound |= OreTypes.Kernite;
                            break;
                    }
                    break;
                case 3: // D
                    Compound |= OreTypes.Veldspar | OreTypes.Scordite | OreTypes.Plagioclase;
                    switch (system.SecurityClassNumber)
                    {
                        case 3:
                            Compound |= OreTypes.Hemorphite;
                            goto case 2;
                        case 2:
                            Compound |= OreTypes.Jaspet;
                            goto case 1;
                        case 1:
                            Compound |= OreTypes.Omber;
                            break;
                    }
                    break;
                case 4: // E
                    Compound |= OreTypes.Veldspar | OreTypes.Scordite | OreTypes.Plagioclase | OreTypes.Omber |
                                OreTypes.Kernite;
                    switch (system.SecurityClassNumber)
                    {
                        case 1:
                            Compound |= OreTypes.Hedbergite;
                            break;
                    }
                    break;
                case 5: // F
                    Compound |= OreTypes.Veldspar | OreTypes.Scordite | OreTypes.Hedbergite | OreTypes.Hemorphite |
                                OreTypes.Omber;
                    switch (system.SecurityClassNumber)
                    {
                        case 7:
                            Compound |= OreTypes.Plagioclase;
                            goto case 6;
                        case 6:
                            Compound |= OreTypes.Mercoxit;
                            goto case 5;
                        case 5:
                            Compound |= OreTypes.Pyroxeres;
                            goto case 4;
                        case 4:
                            Compound |= OreTypes.Arkonor;
                            goto case 3;
                        case 3:
                            Compound |= OreTypes.Bistot;
                            goto case 2;
                        case 2:
                            Compound |= OreTypes.Gneiss;
                            goto case 1;
                        case 1:
                            Compound |= OreTypes.Spodumain;
                            break;
                    }
                    break;
                case 6: // G
                    Compound |= OreTypes.Veldspar | OreTypes.Scordite | OreTypes.Plagioclase | OreTypes.Omber |
                                OreTypes.Kernite;
                    switch (system.SecurityClassNumber)
                    {
                        case 7:
                            Compound |= OreTypes.DarkOchre;
                            goto case 6;
                        case 6:
                            Compound |= OreTypes.Mercoxit;
                            goto case 5;
                        case 5:
                            Compound |= OreTypes.Crokite;
                            goto case 4;
                        case 4:
                            Compound |= OreTypes.Bistot;
                            goto case 3;
                        case 3:
                            Compound |= OreTypes.Spodumain;
                            goto case 2;
                        case 2:
                            Compound |= OreTypes.Pyroxeres;
                            goto case 1;
                        case 1:
                            Compound |= OreTypes.Gneiss;
                            break;
                    }
                    break;
                case 7: // H
                    Compound |= OreTypes.Veldspar | OreTypes.Scordite | OreTypes.Pyroxeres | OreTypes.Hemorphite |
                                OreTypes.Jaspet;
                    switch (system.SecurityClassNumber)
                    {
                        case 7:
                            Compound |= OreTypes.Bistot;
                            goto case 6;
                        case 6:
                            Compound |= OreTypes.Mercoxit;
                            goto case 5;
                        case 5:
                            Compound |= OreTypes.Spodumain;
                            goto case 4;
                        case 4:
                            Compound |= OreTypes.Crokite;
                            goto case 3;
                        case 3:
                            Compound |= OreTypes.Kernite;
                            goto case 2;
                        case 2:
                            Compound |= OreTypes.DarkOchre;
                            goto case 1;
                        case 1:
                            Compound |= OreTypes.Hedbergite;
                            break;
                    }
                    break;
                case 8: // I
                    Compound |= OreTypes.Veldspar | OreTypes.Scordite | OreTypes.Hedbergite | OreTypes.Hemorphite |
                                OreTypes.Omber;
                    switch (system.SecurityClassNumber)
                    {
                        case 7:
                            Compound |= OreTypes.Kernite;
                            goto case 6;
                        case 6:
                            Compound |= OreTypes.Mercoxit;
                            goto case 5;
                        case 5:
                            Compound |= OreTypes.Arkonor;
                            goto case 4;
                        case 4:
                            Compound |= OreTypes.DarkOchre;
                            goto case 3;
                        case 3:
                            Compound |= OreTypes.Gneiss;
                            goto case 2;
                        case 2:
                            Compound |= OreTypes.Spodumain;
                            goto case 1;
                        case 1:
                            Compound |= OreTypes.Jaspet;
                            break;
                    }
                    break;
                case 9: // J
                    Compound |= OreTypes.Veldspar | OreTypes.Scordite | OreTypes.Pyroxeres | OreTypes.Plagioclase |
                                OreTypes.Jaspet;
                    switch (system.SecurityClassNumber)
                    {
                        case 7:
                            Compound |= OreTypes.Arkonor;
                            goto case 6;
                        case 6:
                            Compound |= OreTypes.Mercoxit;
                            goto case 5;
                        case 5:
                            Compound |= OreTypes.Hedbergite;
                            goto case 4;
                        case 4:
                            Compound |= OreTypes.Hemorphite;
                            goto case 3;
                        case 3:
                            Compound |= OreTypes.Bistot;
                            goto case 2;
                        case 2:
                            Compound |= OreTypes.Crokite;
                            goto case 1;
                        case 1:
                            Compound |= OreTypes.DarkOchre;
                            break;
                    }
                    break;
                case 10: // K
                    Compound |= OreTypes.Veldspar | OreTypes.Scordite | OreTypes.Hedbergite | OreTypes.Hemorphite |
                                OreTypes.Omber;
                    switch (system.SecurityClassNumber)
                    {
                        case 7:
                            Compound |= OreTypes.Gneiss;
                            goto case 6;
                        case 6:
                            Compound |= OreTypes.Mercoxit;
                            goto case 5;
                        case 5:
                            Compound |= OreTypes.Arkonor;
                            goto case 4;
                        case 4:
                            Compound |= OreTypes.Bistot;
                            goto case 3;
                        case 3:
                            Compound |= OreTypes.Crokite;
                            goto case 2;
                        case 2:
                            Compound |= OreTypes.Spodumain;
                            goto case 1;
                        case 1:
                            Compound |= OreTypes.DarkOchre;
                            break;
                    }
                    break;
            }

            return Compound;
        }

        public static IceTypes GetIces(EVESystem system)
        {
            IceTypes Compound = IceTypes.None;

            if (system.Security < -0.45) Compound |= IceTypes.Krystallos;
            if (system.Security > -0.65 && system.Security < -0.15) Compound |= IceTypes.Gelidus;
            if (system.Security > -0.45 && system.Security < +0.15) Compound |= IceTypes.DarkGlitter;
            if (system.Security > -0.15 && system.Security < +0.35) Compound |= IceTypes.GlareCrust;

            if (system.ParentRegion.ParentFaction != null)
            {
                switch (system.ParentRegion.ParentFaction.Name)
                {
                    case "Caldari State":
                        Compound |= IceTypes.WhiteGlaze;
                        break;
                    case "Minmatar Republic":
                    case "Ammatar Mandate":
                        Compound |= IceTypes.GlacialMass;
                        break;
                    case "Amarr Empire":
                    case "Khanid Kingdom":
                        Compound |= IceTypes.ClearIcicle;
                        break;
                    case "Gallente Federation":
                        Compound |= IceTypes.BlueIce;
                        break;
                }
            }
            switch (system.ParentRegion.Name)
            {
                case "Vale of the Silent":
                case "Tribute":
                case "Venal":
                case "Geminate":
                case "Deklein":
                case "Tenal":
                case "Branch":
                    Compound |= IceTypes.PristineWhiteGlaze;
                    break;
                case "Detorid":
                case "Wicked Creek":
                case "Cache":
                case "Scalding Pass":
                case "Insmother":
                case "Great Wildlands":
                case "Curse":
                case "Immensea":
                case "Impass":
                case "Feythabolis":
                case "Tenerifis":
                case "Omist":
                    Compound |= IceTypes.SmoothGlacialMass;
                    break;
                case "Catch":
                case "Stain":
                case "Esoteria":
                case "Providence":
                case "Querious":
                case "Paragon Soul":
                case "Delve":
                case "Period Basis":
                    Compound |= IceTypes.EnrichedClearIcicle;
                    break;
                case "Pure Blind":
                case "Syndicate":
                case "Fade":
                case "Cloud Ring":
                case "Outer Ring":
                case "Fountain":
                    Compound |= IceTypes.ThickBlueIce;
                    break;
                case "LQ-0QN":
                case "VU-WU2":
                case "6HL8-L":
                case "7-KXBJ":
                case "87-1CW":
                case "G5KW-3":
                case "H3J8-U":
                case "S-I6VU":
                    break;
            }

            return Compound;
        }

        public static List<string> ListComposition(OreTypes ores)
        {
            var newList = new List<string>();
            foreach (OreTypes ore in Enum.GetValues(typeof (OreTypes)))
                if ((int) ore != 0 && (ore & ores) == ore) newList.Add(ore.ToString());

            return newList;
        }

        public static List<string> ListComposition(IceTypes ices)
        {
            var newList = new List<string>();
            foreach (IceTypes ice in Enum.GetValues(typeof (IceTypes)))
                if ((int) ice != 0 && (ice & ices) == ice) newList.Add(ice.ToString());

            return newList;
        }
    }

    public enum EVEFilterScope
    {
        Systems,
        Stations,
        Agents
    }

    [Serializable]
    public class EVEFilterList : List<EVEFilter>
    {
        public EVEFilterScope Scope = EVEFilterScope.Systems;


        public bool TestSystem(EVESystem system)
        {
            int passed = 0;

            switch (Scope)
            {
                case EVEFilterScope.Systems:
                    foreach (EVEFilter f in this)
                    {
                        if (!f.TestSystem(system)) break;
                        passed++;
                    }
                    if (passed == Count) return true;

                    break;
                case EVEFilterScope.Stations:
                    foreach (EVEStation station in system.Stations)
                    {
                        passed = 0;
                        foreach (EVEFilter f in this)
                        {
                            if (!f.TestStation(station)) break;
                            passed++;
                        }
                        if (passed == Count) return true;
                    }
                    break;
                case EVEFilterScope.Agents:
                    foreach (EVEStation station in system.Stations)
                    {
                        foreach (EVEAgent agent in station)
                        {
                            passed = 0;
                            foreach (EVEFilter f in this)
                            {
                                if (!f.TestAgent(agent)) break;
                                passed++;
                            }
                            if (passed == Count) return true;
                        }
                    }
                    break;
            }
            return false;
        }

        public bool TestStation(EVEStation station)
        {
            int passed = 0;

            switch (Scope)
            {
                case EVEFilterScope.Systems:
                    foreach (EVEFilter f in this)
                    {
                        if (!f.TestSystem(station.ParentSystem)) break;
                        passed++;
                    }
                    if (passed == Count) return true;
                    break;
                case EVEFilterScope.Stations:
                    passed = 0;
                    foreach (EVEFilter f in this)
                    {
                        if (!f.TestStation(station)) break;
                        passed++;
                    }
                    if (passed == Count) return true;
                    break;
                case EVEFilterScope.Agents:
                    for (int i = 0; i < station.Count; i++)
                    {
                        passed = 0;
                        foreach (EVEFilter f in this)
                        {
                            if (!f.TestAgent(station[i])) break;
                            passed++;
                        }
                        if (passed == Count) return true;
                    }
                    break;
            }
            return false;
        }

        public bool TestAgent(EVEAgent agent)
        {
            int passed = 0;

            switch (Scope)
            {
                case EVEFilterScope.Systems:
                    foreach (EVEFilter f in this)
                    {
                        if (!f.TestSystem(agent.ParentStation.ParentSystem)) break;
                        passed++;
                    }
                    if (passed == Count) return true;
                    break;
                case EVEFilterScope.Stations:
                    passed = 0;
                    foreach (EVEFilter f in this)
                    {
                        if (!f.TestStation(agent.ParentStation)) break;
                        passed++;
                    }
                    if (passed == Count) return true;
                    break;
                case EVEFilterScope.Agents:
                    passed = 0;
                    foreach (EVEFilter f in this)
                    {
                        if (!f.TestAgent(agent)) break;
                        passed++;
                    }
                    if (passed == Count) return true;
                    break;
            }
            return false;
        }
    }

    public enum EVEFilterType
    {
        FactionID,
        CorporationID,
        AgentLevel,
        AgentQuality,
        AgentIsLocator,
        AgentDivisionID,
        AgentTypeID,
        StationReprocessing,
        StationManufacturing,
        StationCloning,
        RegionID,
        SystemSecurity,
        SystemAsteroidBeltCount,
        SystemIceFieldCount,
        SystemMoonCount,
        BeltOreType,
        BeltIceType
    }

    public enum EVEFilterCompare
    {
        Equal,
        LargerEqual,
        SmallerEqual,
        Larger,
        Smaller
    }

    [Serializable]
    public class EVEFilter
    {
        public EVEFilterCompare Compare;
        public double DoubleValue;
        public int IntValue;
        public EVEFilterType Type;

        public static string GetCompareString(EVEFilterCompare c)
        {
            switch (c)
            {
                case EVEFilterCompare.Equal:
                    return "=";
                case EVEFilterCompare.Larger:
                    return ">";
                case EVEFilterCompare.LargerEqual:
                    return ">=";
                case EVEFilterCompare.Smaller:
                    return "<";
                case EVEFilterCompare.SmallerEqual:
                    return "<=";
            }
            return "";
        }

        private static bool TestValue(EVEFilterCompare compare, int value, int comparevalue)
        {
            switch (compare)
            {
                case EVEFilterCompare.Equal:
                    return (comparevalue == value);
                case EVEFilterCompare.Larger:
                    return (comparevalue > value);
                case EVEFilterCompare.LargerEqual:
                    return (comparevalue >= value);
                case EVEFilterCompare.Smaller:
                    return (comparevalue < value);
                case EVEFilterCompare.SmallerEqual:
                    return (comparevalue <= value);
            }
            return false;
        }

        private static bool TestValue(EVEFilterCompare compare, double value, double comparevalue)
        {
            switch (compare)
            {
                case EVEFilterCompare.Larger:
                    return (comparevalue > value);
                case EVEFilterCompare.Smaller:
                    return (comparevalue < value);
            }
            return false;
        }


        public bool TestSystem(EVESystem system)
        {
            int count = 0;

            switch (Type)
            {
                case EVEFilterType.FactionID:
                    foreach (EVEStation station in system.Stations)
                    {
                        if (station.ParentCorpNPC != null &&
                            station.ParentCorpNPC.ParentFaction != null &&
                            station.ParentCorpNPC.ParentFaction.ID == IntValue) return true;
                    }
                    return false;
                case EVEFilterType.CorporationID:
                    foreach (EVEStation station in system.Stations)
                    {
                        if (station.ParentCorpNPC.ID == IntValue) return true;
                    }
                    return false;
                case EVEFilterType.AgentLevel:
                    foreach (EVEStation station in system.Stations)
                    {
                        foreach (EVEAgent agent in station)
                            if (TestValue(Compare, IntValue, agent.Level)) return true;
                    }
                    return false;
                case EVEFilterType.AgentQuality:
                    foreach (EVEStation station in system.Stations)
                    {
                        foreach (EVEAgent agent in station)
                            if (TestValue(Compare, IntValue, agent.Quality)) return true;
                    }
                    return false;
                case EVEFilterType.AgentIsLocator:
                    foreach (EVEStation station in system.Stations)
                    {
                        foreach (EVEAgent agent in station)
                            if (agent.Locator) return true;
                    }
                    return false;
                case EVEFilterType.AgentDivisionID:
                    foreach (EVEStation station in system.Stations)
                    {
                        foreach (EVEAgent agent in station)
                            if (agent.Division.ID == IntValue) return true;
                    }
                    return false;
                case EVEFilterType.AgentTypeID:
                    foreach (EVEStation station in system.Stations)
                    {
                        foreach (EVEAgent agent in station)
                            if (agent.AgentType.ID == IntValue) return true;
                    }
                    return false;
                case EVEFilterType.StationReprocessing:
                    foreach (EVEStation station in system.Stations)
                    {
                        IEnumerable<EVEService> services =
                            EVEData.Instance.StationMetaList.GetServiceListFromBitmask(station.ServiceBitmask);
                        foreach (EVEService serv in services)
                        {
                            if (serv.ID == 16) return true;
                        }
                    }
                    return false;
                case EVEFilterType.StationManufacturing:
                    foreach (EVEStation station in system.Stations)
                    {
                        IEnumerable<EVEService> services =
                            EVEData.Instance.StationMetaList.GetServiceListFromBitmask(station.ServiceBitmask);
                        foreach (EVEService serv in services)
                        {
                            if (serv.ID == 8192) return true;
                        }
                    }
                    return false;
                case EVEFilterType.StationCloning:
                    foreach (EVEStation station in system.Stations)
                    {
                        IEnumerable<EVEService> services =
                            EVEData.Instance.StationMetaList.GetServiceListFromBitmask(station.ServiceBitmask);
                        foreach (EVEService serv in services)
                        {
                            if (serv.ID == 512) return true;
                        }
                    }
                    return false;
                case EVEFilterType.RegionID:
                    if (system.ParentRegion.ID == IntValue) return true;
                    return false;
                case EVEFilterType.SystemSecurity:
                    if (TestValue(Compare, DoubleValue, system.Security)) return true;
                    return false;
                case EVEFilterType.SystemAsteroidBeltCount:
                    foreach (EVEPlanet planet in system.Planets)
                        count += planet.AsteroidBeltCount;
                    if (TestValue(Compare, IntValue, count)) return true;
                    return false;
                case EVEFilterType.SystemIceFieldCount:
                    foreach (EVEPlanet planet in system.Planets)
                        count += planet.IceFieldCount;
                    if (TestValue(Compare, IntValue, count)) return true;
                    return false;
                case EVEFilterType.SystemMoonCount:
                    foreach (EVEPlanet planet in system.Planets)
                        count += planet.MoonCount;
                    if (TestValue(Compare, IntValue, count)) return true;
                    return false;
                case EVEFilterType.BeltOreType:
                    EVEMinerals.OreTypes newore = EVEMinerals.GetOres(system);
                    return ((newore & (EVEMinerals.OreTypes) IntValue) == (EVEMinerals.OreTypes) IntValue);
                case EVEFilterType.BeltIceType:
                    EVEMinerals.IceTypes newice = EVEMinerals.GetIces(system);
                    return ((newice & (EVEMinerals.IceTypes) IntValue) == (EVEMinerals.IceTypes) IntValue);
            }
            return false;
        }

        public bool TestStation(EVEStation station)
        {
            int count = 0;
            IEnumerable<EVEService> services;

            switch (Type)
            {
                case EVEFilterType.FactionID:
                    if (station.ParentCorpNPC != null && station.ParentCorpNPC.ParentFaction != null &&
                        station.ParentCorpNPC.ParentFaction.ID == IntValue) return true;
                    return false;
                case EVEFilterType.CorporationID:
                    if (station.ParentCorpNPC != null && station.ParentCorpNPC.ID == IntValue) return true;
                    return false;
                case EVEFilterType.AgentLevel:
                    for (int j = 0; j < station.Count; j++)
                        if (TestValue(Compare, IntValue, station[j].Level)) return true;
                    return false;
                case EVEFilterType.AgentQuality:
                    for (int j = 0; j < station.Count; j++)
                        if (TestValue(Compare, IntValue, station[j].Quality)) return true;
                    return false;
                case EVEFilterType.AgentIsLocator:
                    for (int j = 0; j < station.Count; j++)
                        if (station[j].Locator) return true;
                    return false;
                case EVEFilterType.AgentDivisionID:
                    for (int j = 0; j < station.Count; j++)
                        if (station[j].Division.ID == IntValue) return true;
                    return false;
                case EVEFilterType.AgentTypeID:
                    for (int j = 0; j < station.Count; j++)
                        if (station[j].AgentType.ID == IntValue) return true;
                    return false;
                case EVEFilterType.StationReprocessing:
                    services = EVEData.Instance.StationMetaList.GetServiceListFromBitmask(station.ServiceBitmask);
                    foreach (EVEService serv in services)
                    {
                        if (serv.ID == 16) return true;
                    }
                    return false;
                case EVEFilterType.StationManufacturing:
                    services = EVEData.Instance.StationMetaList.GetServiceListFromBitmask(station.ServiceBitmask);
                    foreach (EVEService serv in services)
                    {
                        if (serv.ID == 8192) return true;
                    }
                    return false;
                case EVEFilterType.StationCloning:
                    services = EVEData.Instance.StationMetaList.GetServiceListFromBitmask(station.ServiceBitmask);
                    foreach (EVEService serv in services)
                    {
                        if (serv.ID == 512) return true;
                    }
                    return false;
                case EVEFilterType.RegionID:
                    if (station.ParentSystem.ParentRegion.ID == IntValue) return true;
                    return false;
                case EVEFilterType.SystemSecurity:
                    if (TestValue(Compare, DoubleValue, station.ParentSystem.Security)) return true;
                    return false;
                case EVEFilterType.SystemAsteroidBeltCount:
                    foreach (EVEPlanet planet in station.ParentSystem.Planets)
                        count += planet.AsteroidBeltCount;
                    if (TestValue(Compare, IntValue, count)) return true;
                    return false;
                case EVEFilterType.SystemIceFieldCount:
                    foreach (EVEPlanet planet in station.ParentSystem.Planets)
                        count += planet.IceFieldCount;
                    if (TestValue(Compare, IntValue, count)) return true;
                    return false;
                case EVEFilterType.SystemMoonCount:
                    foreach (EVEPlanet planet in station.ParentSystem.Planets)
                        count += planet.MoonCount;
                    if (TestValue(Compare, IntValue, count)) return true;
                    return false;
                case EVEFilterType.BeltOreType:
                    EVEMinerals.OreTypes newore = EVEMinerals.GetOres(station.ParentSystem);
                    return ((newore & (EVEMinerals.OreTypes) IntValue) == (EVEMinerals.OreTypes) IntValue);
                case EVEFilterType.BeltIceType:
                    EVEMinerals.IceTypes newice = EVEMinerals.GetIces(station.ParentSystem);
                    return ((newice & (EVEMinerals.IceTypes) IntValue) == (EVEMinerals.IceTypes) IntValue);
            }
            return false;
        }


        public bool TestAgent(EVEAgent agent)
        {
            int count = 0;
            IEnumerable<EVEService> services;

            switch (Type)
            {
                case EVEFilterType.FactionID:
                    if (agent.ParentStation.ParentCorpNPC != null &&
                        agent.ParentStation.ParentCorpNPC.ParentFaction != null &&
                        agent.ParentStation.ParentCorpNPC.ParentFaction.ID == IntValue) return true;
                    return false;
                case EVEFilterType.CorporationID:
                    if (agent.ParentStation.ParentCorpNPC != null &&
                        agent.ParentStation.ParentCorpNPC.ID == IntValue) return true;
                    return false;
                case EVEFilterType.AgentLevel:
                    if (TestValue(Compare, IntValue, agent.Level)) return true;
                    return false;
                case EVEFilterType.AgentQuality:
                    if (TestValue(Compare, IntValue, agent.Quality)) return true;
                    return false;
                case EVEFilterType.AgentIsLocator:
                    if (agent.Locator) return true;
                    return false;
                case EVEFilterType.AgentDivisionID:
                    if (agent.Division.ID == IntValue) return true;
                    return false;
                case EVEFilterType.AgentTypeID:
                    if (agent.AgentType.ID == IntValue) return true;
                    return false;
                case EVEFilterType.StationReprocessing:
                    services =
                        EVEData.Instance.StationMetaList.GetServiceListFromBitmask(agent.ParentStation.ServiceBitmask);
                    foreach (EVEService serv in services)
                    {
                        if (serv.ID == 16) return true;
                    }
                    return false;
                case EVEFilterType.StationManufacturing:
                    services =
                        EVEData.Instance.StationMetaList.GetServiceListFromBitmask(agent.ParentStation.ServiceBitmask);
                    foreach (EVEService serv in services)
                    {
                        if (serv.ID == 8192) return true;
                    }
                    return false;
                case EVEFilterType.StationCloning:
                    services =
                        EVEData.Instance.StationMetaList.GetServiceListFromBitmask(agent.ParentStation.ServiceBitmask);
                    foreach (EVEService serv in services)
                    {
                        if (serv.ID == 512) return true;
                    }
                    return false;
                case EVEFilterType.RegionID:
                    if (agent.ParentStation.ParentSystem.ParentRegion.ID == IntValue) return true;
                    return false;
                case EVEFilterType.SystemSecurity:
                    if (TestValue(Compare, DoubleValue, agent.ParentStation.ParentSystem.Security)) return true;
                    return false;
                case EVEFilterType.SystemAsteroidBeltCount:
                    foreach (EVEPlanet planet in agent.ParentStation.ParentSystem.Planets)
                        count += planet.AsteroidBeltCount;
                    if (TestValue(Compare, IntValue, count)) return true;
                    return false;
                case EVEFilterType.SystemIceFieldCount:
                    foreach (EVEPlanet planet in agent.ParentStation.ParentSystem.Planets)
                        count += planet.IceFieldCount;
                    if (TestValue(Compare, IntValue, count)) return true;
                    return false;
                case EVEFilterType.SystemMoonCount:
                    foreach (EVEPlanet planet in agent.ParentStation.ParentSystem.Planets)
                        count += planet.MoonCount;
                    if (TestValue(Compare, IntValue, count)) return true;
                    return false;
                case EVEFilterType.BeltOreType:
                    EVEMinerals.OreTypes newore = EVEMinerals.GetOres(agent.ParentStation.ParentSystem);
                    return ((newore & (EVEMinerals.OreTypes) IntValue) == (EVEMinerals.OreTypes) IntValue);
                case EVEFilterType.BeltIceType:
                    EVEMinerals.IceTypes newice = EVEMinerals.GetIces(agent.ParentStation.ParentSystem);
                    return ((newice & (EVEMinerals.IceTypes) IntValue) == (EVEMinerals.IceTypes) IntValue);
            }
            return false;
        }
    }
}