using System;
using System.Collections.Generic;
using System.Runtime.Serialization;
using libEveMapNG.Data.Static;
using libEveMapNG.EveAPI;
using libEveMapNG.Utility;

namespace libEveMapNG.Data.Dynamic
{
    [Serializable]
    public static class Singleton
    {
        private static DynamicStore _instance;
        //these two are required.
        public static DynamicStore Instance
        {
            get { return _instance; }
            set { _instance = value; }
        }
    }

    [Serializable]
    public class DynamicStore
    {
        [NonSerialized] private const string STORE_FILE = "dynamic.store";
        [NonSerialized] private Dictionary<SolarSystem, CynoBeacon> _cynobeacons;
        private EveAPIData _eveApiData;
        private List<JumpBridge> _jumpbridges;
        private List<CynoBeacon> _serializedCynoBeaconStore;
        [NonSerialized] private Dictionary<SolarSystem, List<SolarSystem>> _universeGraph;

        public Dictionary<SolarSystem, CynoBeacon> CynoBeacons
        {
            get
            {
                if (_cynobeacons != null)
                    return _cynobeacons;
                _cynobeacons = new Dictionary<SolarSystem, CynoBeacon>();
                return _cynobeacons;
            }
        }

        public List<JumpBridge> JumpBridges
        {
            get
            {
                if (_jumpbridges != null)
                    return _jumpbridges;
                _jumpbridges = new List<JumpBridge>();
                return _jumpbridges;
            }
        }

        public EveAPIData ApiData
        {
            get
            {
                if (_eveApiData != null)
                    return _eveApiData;
                _eveApiData = new EveAPIData();
                return _eveApiData;
            }
        }

        public Dictionary<SolarSystem, List<SolarSystem>> UniverseGraph
        {
            get { return _universeGraph; }
            set { _universeGraph = value; }
        }

        [OnSerializing]
        internal void OnSerializing(StreamingContext context)
        {
            _serializedCynoBeaconStore = new List<CynoBeacon>();
            foreach (KeyValuePair<SolarSystem, CynoBeacon> beacon in _cynobeacons)
            {
                _serializedCynoBeaconStore.Add(beacon.Value);
            }
        }

        [OnDeserialized]
        internal void OnDeserialized(StreamingContext context)
        {
            _cynobeacons = new Dictionary<SolarSystem, CynoBeacon>();
            foreach (CynoBeacon beacon in _serializedCynoBeaconStore)
            {
                _cynobeacons.Add(beacon.System, beacon);
            }
        }

        #region Singleton Handling

        public static DynamicStore Instance
        {
            get { return Singleton.Instance; }
            private set { Singleton.Instance = value; }
        }

        public static void WriteToFile()
        {
            Serializer.WriteToAppData(STORE_FILE, Singleton.Instance);
        }

        public static void ReadFromFile()
        {
            try
            {
                Instance = Serializer.ReadFromAppData<DynamicStore>(STORE_FILE);
            }
            catch (SerializationException)
            {
                Files.DeleteFile("Dynamic.store");
            }
            catch (Exception ex)
            {
                Instance = new DynamicStore();
            }
        }

        #endregion
    }
}