using System;
using System.Collections.Generic;
using System.Drawing;
using System.Linq;
using libEveMapNG.Types;
using libEveMapNG.Utility;

namespace libEveMapNG.Data.Static
{
    [Serializable]
    public class SolarSystem
    {
        public RelativePosition FlatLabel;
        public Point OrganicCoordinates;
        private Constellation _constellation;
        private Region _region;
        public int ConstellationID { get; set; }
        public int RegionID { get; set; }
        public int ID { get; set; }
        public string Name { get; set; }
        public double Security { get; set; }
        public string SecurityClass { get; set; }
        public PointF3D Coordinates { get; set; }
        public List<StarGate> StarGates { get; set; }
        public List<Planet> Planets { get; set; }
        public List<Station> Stations { get; set; }

        public Constellation Constellation
        {
            get
            {
                if (_constellation == null)
                    _constellation = Constellation.GetByID(ConstellationID);
                return _constellation;
            }
        }

        public Region Region
        {
            get
            {
                if (_region == null)
                {
                    foreach (Region r in StaticStore.Instance.Regions)
                    {
                        if (r.ID == RegionID)
                        {
                            _region = r;
                            break;
                        }
                    }
                }
                return _region;
            }
        }

        public static SolarSystem GetByID(int ID)
        {
            return StaticStore.Systems[ID];
        }

        public static bool operator ==(SolarSystem a, SolarSystem b)
        {
            // If both are null, or both are same instance, return true.
            if (ReferenceEquals(a, b))
                return true;
            // If one is null, but not both, return false.
            if (((object) a == null) || ((object) b == null))
                return false;
            return a.ID == b.ID;
        }

        public static bool operator !=(SolarSystem a, SolarSystem b)
        {
            return !(a == b);
        }

        public static int GetIDByName(string name)
        {
            SolarSystem mySystem = GetByName(name);
            if (mySystem == null)
                throw new Exception(String.Format("solar system {0} does not exist.", name));
            return mySystem.ID;
        }

        public static Dictionary<int, SolarSystem> GetSystemsInRange(SolarSystem system, int JDC,
                                                                     CapitalShips.Ship shipType)
        {
            float jumpRange = JumpDrive.GetRange(JDC, shipType);

            IEnumerable<KeyValuePair<int, SolarSystem>> systemsInRange = from solarSystem in StaticStore.Systems
                                                                         where
                                                                             solarSystem.Value.Security < 0.45f &&
                                                                             JumpDrive.GetDistance(solarSystem.Value,
                                                                                                   system) <= jumpRange
                                                                         select solarSystem;
            return systemsInRange.ToDictionary(n => n.Value.ID, n => n.Value);
        }

        public static SolarSystem GetByName(string name)
        {
            if (string.IsNullOrEmpty(name.Trim()))
                return null;
            SolarSystem system = (from solarSystem in StaticStore.Systems
                                  where solarSystem.Value.Name.ToLower() == name.ToLower()
                                  select solarSystem).Single().Value;
            return system;
        }

        private bool Equals(SolarSystem obj)
        {
            if (ReferenceEquals(null, obj))
                return false;
            if (ReferenceEquals(this, obj))
                return true;
            return obj.ID == ID;
        }

        public override bool Equals(object obj)
        {
            if (ReferenceEquals(null, obj))
                return false;
            if (ReferenceEquals(this, obj))
                return true;
            if (obj.GetType() != typeof (SolarSystem))
                return false;
            return Equals((SolarSystem) obj);
        }

        public override int GetHashCode()
        {
            return ID;
        }
    }
}