﻿using System;
using System.Collections.Generic;

namespace libEveMapNG.Data.Static
{
    [Serializable]
    public class Starbase
    {
        public Dictionary<string, StarbaseFuel> Fuel;
        public DateTime LastUpdate;
        public int Moon;
        public string Name;
        public int Planet;
        public int SolarSystemID;
        public int TypeID;

        public TimeSpan RunTime
        {
            get
            {
                TimeSpan minTime = TimeSpan.Zero;
                foreach (KeyValuePair<string, StarbaseFuel> fuel in Fuel)
                {
                    TimeSpan runTime = TimeSpan.FromHours(fuel.Value.Quantity/fuel.Value.QuantityUsedPerHour);
                    if (minTime == TimeSpan.Zero || minTime < runTime)
                        minTime = runTime;
                }
                return minTime;
            }
        }

        public StarbaseFuel GetFuelByID(int ID)
        {
            foreach (KeyValuePair<string, StarbaseFuel> pair in Fuel)
            {
                if (pair.Value.TypeId == ID)
                    return pair.Value;
            }
            return new StarbaseFuel();
        }

        public static Starbase Copy(Starbase starbase)
        {
            Starbase Temp = new Starbase();
            Temp.LastUpdate = starbase.LastUpdate;
            Temp.Moon = starbase.Moon;
            Temp.Name = starbase.Name;
            Temp.Planet = starbase.Planet;
            Temp.SolarSystemID = starbase.SolarSystemID;
            Temp.TypeID = starbase.TypeID;
            Temp.Fuel = new Dictionary<string, StarbaseFuel>();
            foreach (KeyValuePair<string, StarbaseFuel> pair in starbase.Fuel)
            {
                Temp.Fuel.Add(pair.Key, StarbaseFuel.Copy(pair.Value));
            }

            return (Starbase) starbase.MemberwiseClone();
        }
    }

    [Serializable]
    public class StarbaseFuel
    {
        public string Name;
        public int Quantity;
        public int QuantityUsedPerHour;
        public int TypeId;
        public float Volume;

        public static StarbaseFuel Copy(StarbaseFuel starbaseFuel)
        {
            StarbaseFuel Temp = new StarbaseFuel();
            Temp.TypeId = starbaseFuel.TypeId;
            Temp.Quantity = starbaseFuel.Quantity;
            Temp.QuantityUsedPerHour = starbaseFuel.QuantityUsedPerHour;
            Temp.Name = starbaseFuel.Name;
            Temp.Volume = starbaseFuel.Volume;
            return Temp;
        }
    }
}