using System;
using System.Collections.Generic;
using System.Diagnostics;
using libEveMapNG.Utility;

namespace libEveMapNG.Data.Static
{
    [Serializable]
    public static class Singleton
    {
        private static StaticStore instance;

        public static StaticStore Instance
        {
            get { return instance; }
            set { instance = value; }
        }
    }

    [Serializable]
    public class StaticStore
    {
        [NonSerialized] private const string STORE_FILE = "data.store";

        //Holds everything the entire universe
#pragma warning disable 649
        private Dictionary<int, StationType> _stationTypes;
        private List<Region> _regions;
        private Dictionary<int, Starbase> _starbaseTypes;
        private List<StarGate> _starGates;

        #region Singleton Handling

        static StaticStore()
        {
            Systems = new Dictionary<int, SolarSystem>();
            Constellations = new Dictionary<int, Constellation>();
            Stations = new List<Station>();
        }

        public static StaticStore Instance
        {
            get { return Singleton.Instance; }
            set
            {
                Singleton.Instance = value;
                RebuildHashTables();
            }
        }

        public static void WriteToFile()
        {
            Serializer.WriteToFile(STORE_FILE, Singleton.Instance);
        }

        public static void ReadFromFile()
        {
            Debug.WriteLine("Reading store...");
            Instance = Serializer.ReadFromFile<StaticStore>(STORE_FILE);
        }

        #endregion

#pragma warning restore 649
        public List<Region> Regions
        {
            get { return _regions; }
            set { _regions = value; }
        }


        public Dictionary<int, Starbase> StarbaseTypes
        {
            get { return _starbaseTypes; }
            set { _starbaseTypes = value; }
        }


        public List<StarGate> StarGates
        {
            get { return _starGates; }
            set { _starGates = value; }
        }

        public static Dictionary<int, SolarSystem> Systems { get; private set; }
        public static Dictionary<int, Constellation> Constellations { get; private set; }
        public static List<Station> Stations { get; private set; }

        public Dictionary<int, StationType> StationTypes
        {
            get { return _stationTypes; }
            set { _stationTypes = value; }
        }

        public static void RebuildHashTables()
        {
            Constellations.Clear();
            Systems.Clear();
            Stations.Clear();
            foreach (Region region2 in Instance.Regions)
            {
                if (region2.Constellations == null)
                    continue;
                foreach (Constellation constellation2 in region2.Constellations)
                {
                    Constellations.Add(constellation2.ID, constellation2);
                    if (constellation2.Systems == null)
                        continue;
                    foreach (SolarSystem system2 in constellation2.Systems)
                    {
                        Systems.Add(system2.ID, system2);
                        if (system2.Stations != null)
                        {
                            foreach (Station station in system2.Stations)
                                Stations.Add(station);
                        }
                    }
                }
            }
        }
    }
}