using System;
using System.Drawing;

namespace libEveMapNG.Utility
{
    public class Bezier
    {
        public static void DrawBezier(Graphics g, Pen pen, PointF start, PointF end, float nodrawRadius,
                                      float lengthfraction, double angle)
        {
            float drawPosX1 = start.X, drawPosY1 = -start.Y, drawPosX2 = end.X, drawPosY2 = -end.Y;
            float dx = drawPosX2 - drawPosX1;
            float dy = drawPosY2 - drawPosY1;
            if (dx < 0)
            {
                drawPosX1 = end.X;
                drawPosY1 = -end.Y;
                drawPosX2 = start.X;
                drawPosY2 = -start.Y;
                dx = drawPosX2 - drawPosX1;
                dy = drawPosY2 - drawPosY1;
            }
            float length = (float) (Math.Sqrt(dx*dx + dy*dy));
            double alpha = (float) (Math.Acos(dx/length));
            if (dy < 0) // adjust for the right quadrant
            {
                if (dx > 0) alpha = -alpha;
                else alpha = Math.PI + Math.PI - alpha;
            }
            float bx1 = drawPosX1 + length/lengthfraction*(float) Math.Cos(alpha + angle);
            float by1 = drawPosY1 + length/lengthfraction*(float) Math.Sin(alpha + angle);
            float bx2 = drawPosX2 - length/lengthfraction*(float) Math.Cos(alpha - angle);
            float by2 = drawPosY2 - length/lengthfraction*(float) Math.Sin(alpha - angle);
            drawPosX1 += nodrawRadius*(float) Math.Cos(alpha);
            drawPosY1 += nodrawRadius*(float) Math.Sin(alpha);
            drawPosX2 -= nodrawRadius*(float) Math.Cos(alpha);
            drawPosY2 -= nodrawRadius*(float) Math.Sin(alpha);
            PointF src = new PointF(drawPosX1, -drawPosY1);
            PointF bezier1 = new PointF(bx1, -by1);
            PointF bezier2 = new PointF(bx2, -by2);
            PointF dest = new PointF(drawPosX2, -drawPosY2);
            g.DrawBezier(pen, src, bezier1, bezier2, dest);
        }
    }
}