using System;
using System.Runtime.InteropServices;

namespace libEveMapNG.Utility
{
    public class Timer
    {
        #region Native Imports

        [DllImport("kernel32.dll")]
        private static extern bool QueryPerformanceCounter(out long lpPerformanceCount);

        [DllImport("kernel32.dll")]
        private static extern bool QueryPerformanceFrequency(out long lpFrequency);

        #endregion

        #region Fields

        private readonly long _frequency;
        private long _startTime;

        #endregion

        #region Constructor

        public Timer()
        {
            if (!QueryPerformanceFrequency(out _frequency))
                throw new InvalidOperationException();
        }

        #endregion

        public long CurrentTicks
        {
            get
            {
                long ret;
                if (!QueryPerformanceCounter(out ret))
                    throw new InvalidOperationException();

                return ret;
            }
        }

        public double CurrentMilliseconds
        {
            get { return (CurrentTicks/(double) _frequency)*1000.0; }
        }

        public void Start()
        {
            _startTime = CurrentTicks;
        }

        public double Stop()
        {
            long endTime = CurrentTicks;
            return (endTime - _startTime)/(double) _frequency;
        }
    }
}