namespace libeveapi
{
    /// <summary></summary>
    public class SkillTree : ApiResponse
    {
        #region AttributeType enum

        /// <summary>
        ///   Different types of character attributes
        /// </summary>
        public enum AttributeType
        {
            /// <summary>
            ///   The memory attribute
            /// </summary>
            Memory,
            /// <summary>
            ///   The Willpower attribute
            /// </summary>
            Willpower,
            /// <summary>
            ///   The Intelligence attribute
            /// </summary>
            Intelligence,
            /// <summary>
            ///   The Perception attribute
            /// </summary>
            Perception,
            /// <summary>
            ///   The Charisma attribute
            /// </summary>
            Charisma,
            /// <summary>
            ///   What is this attribute?
            /// </summary>
            Unknown
        }

        #endregion

        private SkillGroup[] skillGroups = new SkillGroup[0];
        private Skill[] skills = new Skill[0];

        /// <summary></summary>
        public Skill[] Skills
        {
            get { return skills; }
            set { skills = value; }
        }

        public SkillGroup[] SkillGroups
        {
            get { return skillGroups; }
            set { skillGroups = value; }
        }

        private static AttributeType mapStringToAttributeType(string s)
        {
            switch (s)
            {
                case "memory":
                    return AttributeType.Memory;

                case "willpower":
                    return AttributeType.Willpower;

                case "intelligence":
                    return AttributeType.Intelligence;

                case "perception":
                    return AttributeType.Perception;

                case "charisma":
                    return AttributeType.Charisma;
            }

            return AttributeType.Unknown;
        }

        #region Nested type: RequiredSkill

        /// <summary>
        ///   A skill required to train the parent skill
        /// </summary>
        public class RequiredSkill
        {
            /// <summary>
            ///   The unique identifier of this skill type.
            /// </summary>
            public int TypeId { get; set; }

            /// <summary>
            ///   This skill level is required to train the parent skill.
            /// </summary>
            public int SkillLevel { get; set; }
        }

        #endregion

        #region Nested type: Skill

        /// <summary>
        ///   Information about a EVE Skill
        /// </summary>
        public class Skill
        {
            private RequiredSkill[] requiredSkills = new RequiredSkill[0];
            private SkillBonus[] skillBonuses = new SkillBonus[0];

            /// <summary>
            ///   Name of the skill
            /// </summary>
            public string TypeName { get; set; }

            /// <summary>
            ///   Unique identifier for the group this skill belongs to
            /// </summary>
            public int GroupId { get; set; }

            /// <summary>
            ///   Unique identifier for this skill type
            /// </summary>
            public int TypeId { get; set; }

            /// <summary>
            ///   information about the skill
            /// </summary>
            public string Description { get; set; }

            /// <summary>
            ///   The training time multiplier of the skill
            /// </summary>
            public int Rank { get; set; }

            /// <summary>
            ///   The skills required to train this skill
            /// </summary>
            public RequiredSkill[] RequiredSkills
            {
                get { return requiredSkills; }
                set { requiredSkills = value; }
            }

            /// <summary>
            ///   The attrbute that has the most effect on the amount of time required to train this skill.
            /// </summary>
            public AttributeType PrimaryAttribute { get; set; }

            /// <summary>
            ///   The attribute that has the second most effect of the amount of time required to train this skill.
            /// </summary>
            public AttributeType SecondaryAttribute { get; set; }

            /// <summary>
            ///   The bonuses gained from training this skill.
            /// </summary>
            public SkillBonus[] SkillBonuses
            {
                get { return skillBonuses; }
                set { skillBonuses = value; }
            }
        }

        #endregion

        #region Nested type: SkillBonus

        /// <summary>
        ///   A bonus gained from training the parent skill
        /// </summary>
        public class SkillBonus
        {
            /// <summary>
            ///   Description of the bonus
            /// </summary>
            public string BonusType { get; set; }

            /// <summary>
            ///   The amount of bonus gained from each level of the skill trained.
            /// </summary>
            public double BonusValue { get; set; }
        }

        #endregion

        #region Nested type: SkillGroup

        /// <summary>
        ///   A group of related skills
        /// </summary>
        public class SkillGroup
        {
            /// <summary>
            ///   The name of this skill group.
            /// </summary>
            public string GroupName { get; set; }

            /// <summary>
            ///   The unique identifier for this skill group.
            /// </summary>
            public int GroupId { get; set; }
        }

        #endregion
    }
}