namespace libeveapi
{
    /// <summary>
    ///   Shows the settings and fuel status of a starbase
    /// </summary>
    public class StarbaseDetail : ApiResponse
    {
        private FuelListItem[] fuelList = new FuelListItem[0];

        /// <summary></summary>
        public string UsageFlags { get; set; }

        /// <summary></summary>
        public string DeployFlags { get; set; }

        /// <summary>
        ///   Allow corporation members inside the force field
        /// </summary>
        public bool AllowCorporationMembers { get; set; }

        /// <summary>
        ///   Allow alliance members inside the force field
        /// </summary>
        public bool AllowAllianceMembers { get; set; }

        /// <summary>
        ///   Is the starbase claiming sovernty (only in 0.0 space)
        /// </summary>
        public bool ClaimSovereignty { get; set; }

        /// <summary>
        ///   Shoot on standing drop
        /// </summary>
        public bool OnStandingDropEnabled { get; set; }

        /// <summary>
        ///   What target standing makes them a valid target
        /// </summary>
        public string OnStandingDropStanding { get; set; }

        /// <summary>
        ///   Shoot on status drop
        /// </summary>
        public bool OnStatusDropEnabled { get; set; }

        /// <summary>
        ///   What target security make them a valid target
        /// </summary>
        public string OnStatusDropStanding { get; set; }

        /// <summary>
        ///   Shoot on target agression
        /// </summary>
        public bool OnAgressionEnabled { get; set; }

        /// <summary>
        ///   Shoot if at war with target
        /// </summary>
        public bool OnCorporationWarEnabled { get; set; }

        /// <summary></summary>
        public FuelListItem[] FuelList
        {
            get { return fuelList; }
            set { fuelList = value; }
        }

        #region Nested type: FuelListItem

        /// <summary>
        ///   Represents a type of fuel present in the starbase
        /// </summary>
        public class FuelListItem
        {
            /// <summary>
            ///   Type Id of the fuel
            /// </summary>
            public int TypeId { get; set; }

            /// <summary>
            ///   Number of units of the fuel remaining
            /// </summary>
            public long Quantity { get; set; }
        }

        #endregion
    }
}