using System;

namespace libeveapi
{
    /// <summary>
    ///   Represents a list of all the starbases owned by the specified corporation
    /// </summary>
    public class StarbaseList : ApiResponse
    {
        #region StarbaseState enum

        /// <summary>
        ///   This represents the current state of the starbase, of which there are
        ///   four different valid states.
        /// </summary>
        public enum StarbaseState
        {
            /// <summary>
            ///   Starbase is anchored or offline
            ///   if offline then it went offline at StateTimestamp
            /// </summary>
            AnchoredOrOffline = 1,

            /// <summary>
            ///   Starbase is in the process of coming online
            ///   Will be online at time OnlineTimestamp
            /// </summary>
            Onlining = 2,

            /// <summary>
            ///   Starbase in in reinforced mode
            ///   Until time StateTimestamp
            /// </summary>
            Reinforced = 3,

            /// <summary>
            ///   Starbase is online
            ///   Continuously since time OnlineTimestamp
            /// </summary>
            Online = 4
        }

        #endregion

        private StarbaseListItem[] starbaseListItems = new StarbaseListItem[0];

        /// <summary>
        ///   List of all the starbases owned by this corporation
        /// </summary>
        public StarbaseListItem[] StarbaseListItems
        {
            get { return starbaseListItems; }
            set { starbaseListItems = value; }
        }

        #region Nested type: StarbaseListItem

        /// <summary>
        ///   Represents a single starbase in the starbase list
        /// </summary>
        public class StarbaseListItem
        {
            /// <summary>
            ///   Unique identifier for this starbase
            /// </summary>
            public int ItemId { get; set; }

            /// <summary>
            ///   Control tower type id
            /// </summary>
            public int TypeId { get; set; }

            /// <summary>
            ///   The id of the system where the starbase is located
            /// </summary>
            public int LocationId { get; set; }

            /// <summary>
            ///   The id of the moon where the starbase is located
            /// </summary>
            public int MoonId { get; set; }

            /// <summary>
            ///   See
            ///   <see cref="StarbaseState" />
            ///   for full descriptions of each starbase state
            /// </summary>
            public StarbaseState State { get; set; }

            /// <summary>
            ///   See
            ///   <see cref="StarbaseState" />
            ///   for the potential meanings of StateTimestamp
            /// </summary>
            public DateTime StateTimestamp { get; set; }

            /// <summary>
            ///   See
            ///   <see cref="StarbaseState" />
            ///   for the potential meanings of OnlineTimestamp
            /// </summary>
            public DateTime OnlineTimestamp { get; set; }

            /// <summary>
            ///   StateTimestamp in local time
            /// </summary>
            public DateTime StateTimestampLocal { get; set; }

            /// <summary>
            ///   OnlineTimestamp in local time
            /// </summary>
            public DateTime OnlineTimestampLocal { get; set; }
        }

        #endregion
    }
}