using System;

namespace libeveapi
{
    /// <summary>
    ///   Returns market transactions for a character.
    ///   http://wiki.eve-dev.net/APIv2_Char_MarketTransactions_XML
    /// </summary>
    public class WalletTransactions : ApiResponse
    {
        private WalletTransactionItem[] walletTransactionItems = new WalletTransactionItem[0];

        /// <summary></summary>
        public WalletTransactionItem[] WalletTransactionItems
        {
            get { return walletTransactionItems; }
            set { walletTransactionItems = value; }
        }

        #region Nested type: WalletTransactionItem

        /// <summary>
        ///   Contains information about one wallet transaction
        /// </summary>
        public class WalletTransactionItem
        {
            /// <summary>
            ///   This is the date and time when the transaction took place in ccp time
            /// </summary>
            public DateTime TransactionDateTime { get; set; }

            /// <summary>
            ///   This is the date and time when the transaction took place in local time
            /// </summary>
            public DateTime TransactionDateTimeLocal { get; set; }

            /// <summary>
            ///   This is the transactionId that is assigned to the transaction
            /// </summary>
            public int TransactionId { get; set; }

            /// <summary>
            ///   This is the quantity of the item
            /// </summary>
            public int Quantity { get; set; }

            /// <summary>
            ///   This is the name of the item in the transaction
            /// </summary>
            public string TypeName { get; set; }

            /// <summary>
            ///   This is the typeId of the item referenced in the transaction
            /// </summary>
            public int TypeId { get; set; }

            /// <summary>
            ///   This is the price of the item in the transaction
            /// </summary>
            public double Price { get; set; }

            /// <summary>The client's Id</summary>
            public int ClientId { get; set; }

            /// <summary>
            ///   The client's name
            /// </summary>
            public string ClientName { get; set; }

            /// <summary>
            ///   The character who initiated the transaction's id 
            ///   This is only present when viewing corp transactions, otherwise
            ///   it is assumed to be the character accessing the data
            /// </summary>
            public int CharacterId { get; set; }

            /// <summary>
            ///   The character who initiated the transaction's name 
            ///   This is only present when viewing corp transactions, otherwise
            ///   it is assumed to be the character accessing the data
            /// </summary>
            public string CharacterName { get; set; }

            /// <summary>
            ///   The Id of the station where the transaction took place
            /// </summary>
            public int StationId { get; set; }

            /// <summary>
            ///   The name of the station where the transaction took place
            /// </summary>
            public string StationName { get; set; }

            /// <summary>
            ///   This is the type of transaction type, sell or buy
            /// </summary>
            public string TransactionType { get; set; }

            /// <summary>
            ///   This is who the transaction was for (personal or corporation)
            /// </summary>
            public string TransactionFor { get; set; }
        }

        #endregion
    }

    /// <summary>
    ///   If the transaction is a corporation or character transaction
    /// </summary>
    public enum WalletTransactionListType
    {
        /// <summary>
        ///   A corporation transaction
        /// </summary>
        Corporation,
        /// <summary>
        ///   A character transaction
        /// </summary>
        Character
    }
}